# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sarus_data_spec/protobuf/statistics.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='sarus_data_spec/protobuf/statistics.proto',
  package='sarus_data_spec',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n)sarus_data_spec/protobuf/statistics.proto\x12\x0fsarus_data_spec\"\xf6\x14\n\nStatistics\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x30\n\x04null\x18\x03 \x01(\x0b\x32 .sarus_data_spec.Statistics.NullH\x00\x12\x30\n\x04unit\x18\x04 \x01(\x0b\x32 .sarus_data_spec.Statistics.UnitH\x00\x12\x36\n\x07\x62oolean\x18\x05 \x01(\x0b\x32#.sarus_data_spec.Statistics.BooleanH\x00\x12\x36\n\x07integer\x18\x06 \x01(\x0b\x32#.sarus_data_spec.Statistics.IntegerH\x00\x12\x30\n\x04\x65num\x18\x07 \x01(\x0b\x32 .sarus_data_spec.Statistics.EnumH\x00\x12\x32\n\x05\x66loat\x18\x08 \x01(\x0b\x32!.sarus_data_spec.Statistics.FloatH\x00\x12\x30\n\x04text\x18\t \x01(\x0b\x32 .sarus_data_spec.Statistics.TextH\x00\x12\x32\n\x05\x62ytes\x18\n \x01(\x0b\x32!.sarus_data_spec.Statistics.BytesH\x00\x12\x34\n\x06struct\x18\x0b \x01(\x0b\x32\".sarus_data_spec.Statistics.StructH\x00\x12\x32\n\x05union\x18\x0c \x01(\x0b\x32!.sarus_data_spec.Statistics.UnionH\x00\x12\x38\n\x08optional\x18\r \x01(\x0b\x32$.sarus_data_spec.Statistics.OptionalH\x00\x12\x30\n\x04list\x18\x0e \x01(\x0b\x32 .sarus_data_spec.Statistics.ListH\x00\x12\x32\n\x05\x61rray\x18\x0f \x01(\x0b\x32!.sarus_data_spec.Statistics.ArrayH\x00\x12\x38\n\x08\x64\x61tetime\x18\x10 \x01(\x0b\x32$.sarus_data_spec.Statistics.DatetimeH\x00\x12,\n\x02id\x18\x11 \x01(\x0b\x32\x1e.sarus_data_spec.Statistics.IdH\x00\x12>\n\x0b\x63onstrained\x18\x12 \x01(\x0b\x32\'.sarus_data_spec.Statistics.ConstrainedH\x00\x12?\n\nproperties\x18\x02 \x03(\x0b\x32+.sarus_data_spec.Statistics.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a*\n\x04Null\x12\x0c\n\x04size\x18\x01 \x01(\x03\x12\x14\n\x0cmultiplicity\x18\x02 \x01(\x02\x1a*\n\x04Unit\x12\x0c\n\x04size\x18\x01 \x01(\x03\x12\x14\n\x0cmultiplicity\x18\x02 \x01(\x02\x1a\x62\n\x07\x42oolean\x12\x33\n\x0c\x64istribution\x18\x01 \x01(\x0b\x32\x1d.sarus_data_spec.Distribution\x12\x0c\n\x04size\x18\x02 \x01(\x03\x12\x14\n\x0cmultiplicity\x18\x03 \x01(\x02\x1a(\n\x02Id\x12\x0c\n\x04size\x18\x01 \x01(\x03\x12\x14\n\x0cmultiplicity\x18\x02 \x01(\x02\x1a\x62\n\x07Integer\x12\x33\n\x0c\x64istribution\x18\x01 \x01(\x0b\x32\x1d.sarus_data_spec.Distribution\x12\x0c\n\x04size\x18\x02 \x01(\x03\x12\x14\n\x0cmultiplicity\x18\x03 \x01(\x02\x1a_\n\x04\x45num\x12\x33\n\x0c\x64istribution\x18\x01 \x01(\x0b\x32\x1d.sarus_data_spec.Distribution\x12\x0c\n\x04size\x18\x02 \x01(\x03\x12\x14\n\x0cmultiplicity\x18\x03 \x01(\x02\x1a`\n\x05\x46loat\x12\x33\n\x0c\x64istribution\x18\x01 \x01(\x0b\x32\x1d.sarus_data_spec.Distribution\x12\x0c\n\x04size\x18\x02 \x01(\x03\x12\x14\n\x0cmultiplicity\x18\x03 \x01(\x02\x1ap\n\x04Text\x12\x33\n\x0c\x64istribution\x18\x01 \x01(\x0b\x32\x1d.sarus_data_spec.Distribution\x12\x0f\n\x07\x65xample\x18\x02 \x01(\t\x12\x0c\n\x04size\x18\x03 \x01(\x03\x12\x14\n\x0cmultiplicity\x18\x04 \x01(\x02\x1a+\n\x05\x42ytes\x12\x0c\n\x04size\x18\x01 \x01(\x03\x12\x14\n\x0cmultiplicity\x18\x02 \x01(\x02\x1a\xbc\x01\n\x06Struct\x12\x38\n\x06\x66ields\x18\x01 \x03(\x0b\x32(.sarus_data_spec.Statistics.Struct.Field\x12\x0c\n\x04size\x18\x02 \x01(\x03\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x14\n\x0cmultiplicity\x18\x04 \x01(\x02\x1a\x46\n\x05\x46ield\x12\x0c\n\x04name\x18\x01 \x01(\t\x12/\n\nstatistics\x18\x02 \x01(\x0b\x32\x1b.sarus_data_spec.Statistics\x1a\xba\x01\n\x05Union\x12\x37\n\x06\x66ields\x18\x01 \x03(\x0b\x32\'.sarus_data_spec.Statistics.Union.Field\x12\x0c\n\x04size\x18\x02 \x01(\x03\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x14\n\x0cmultiplicity\x18\x04 \x01(\x02\x1a\x46\n\x05\x46ield\x12\x0c\n\x04name\x18\x01 \x01(\t\x12/\n\nstatistics\x18\x02 \x01(\x0b\x32\x1b.sarus_data_spec.Statistics\x1am\n\x08Optional\x12/\n\nstatistics\x18\x01 \x01(\x0b\x32\x1b.sarus_data_spec.Statistics\x12\x0c\n\x04size\x18\x02 \x01(\x03\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x14\n\x0cmultiplicity\x18\x04 \x01(\x02\x1a\x90\x01\n\x04List\x12/\n\nstatistics\x18\x01 \x01(\x0b\x32\x1b.sarus_data_spec.Statistics\x12\x33\n\x0c\x64istribution\x18\x02 \x01(\x0b\x32\x1d.sarus_data_spec.Distribution\x12\x0c\n\x04size\x18\x03 \x01(\x03\x12\x14\n\x0cmultiplicity\x18\x04 \x01(\x02\x1a\x92\x01\n\x05\x41rray\x12/\n\nstatistics\x18\x01 \x01(\x0b\x32\x1b.sarus_data_spec.Statistics\x12\x34\n\rdistributions\x18\x02 \x03(\x0b\x32\x1d.sarus_data_spec.Distribution\x12\x0c\n\x04size\x18\x03 \x01(\x03\x12\x14\n\x0cmultiplicity\x18\x04 \x01(\x02\x1a\x63\n\x08\x44\x61tetime\x12\x33\n\x0c\x64istribution\x18\x01 \x01(\x0b\x32\x1d.sarus_data_spec.Distribution\x12\x0c\n\x04size\x18\x02 \x01(\x03\x12\x14\n\x0cmultiplicity\x18\x03 \x01(\x02\x1a\x62\n\x0b\x43onstrained\x12/\n\nstatistics\x18\x01 \x01(\x0b\x32\x1b.sarus_data_spec.Statistics\x12\x0c\n\x04size\x18\x02 \x01(\x03\x12\x14\n\x0cmultiplicity\x18\x03 \x01(\x02\x42\x0c\n\nstatistics\"\x93\x07\n\x0c\x44istribution\x12\x36\n\x06\x64ouble\x18\x01 \x01(\x0b\x32$.sarus_data_spec.Distribution.DoubleH\x00\x12\x38\n\x07integer\x18\x02 \x01(\x0b\x32%.sarus_data_spec.Distribution.IntegerH\x00\x12\x38\n\x07\x62oolean\x18\x03 \x01(\x0b\x32%.sarus_data_spec.Distribution.BooleanH\x00\x12\x32\n\x04\x65num\x18\x04 \x01(\x0b\x32\".sarus_data_spec.Distribution.EnumH\x00\x12\x41\n\nproperties\x18\x05 \x03(\x0b\x32-.sarus_data_spec.Distribution.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x8b\x01\n\x06\x44ouble\x12:\n\x06points\x18\x01 \x03(\x0b\x32*.sarus_data_spec.Distribution.Double.Point\x12\x0b\n\x03min\x18\x02 \x01(\x01\x12\x0b\n\x03max\x18\x03 \x01(\x01\x1a+\n\x05Point\x12\r\n\x05value\x18\x02 \x01(\x01\x12\x13\n\x0bprobability\x18\x03 \x01(\x01\x1a\x8d\x01\n\x07Integer\x12;\n\x06points\x18\x01 \x03(\x0b\x32+.sarus_data_spec.Distribution.Integer.Point\x12\x0b\n\x03min\x18\x02 \x01(\x03\x12\x0b\n\x03max\x18\x03 \x01(\x03\x1a+\n\x05Point\x12\r\n\x05value\x18\x01 \x01(\x03\x12\x13\n\x0bprobability\x18\x02 \x01(\x01\x1a\x81\x01\n\x07\x42oolean\x12;\n\x06points\x18\x01 \x03(\x0b\x32+.sarus_data_spec.Distribution.Boolean.Point\x1a\x39\n\x05Point\x12\x0c\n\x04name\x18\x01 \x01(\x08\x12\r\n\x05value\x18\x02 \x01(\x03\x12\x13\n\x0bprobability\x18\x03 \x01(\x01\x1a{\n\x04\x45num\x12\x38\n\x06points\x18\x01 \x03(\x0b\x32(.sarus_data_spec.Distribution.Enum.Point\x1a\x39\n\x05Point\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01\x12\x13\n\x0bprobability\x18\x03 \x01(\x01\x42\x0e\n\x0c\x64istributionb\x06proto3'
)




_STATISTICS_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='sarus_data_spec.Statistics.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='sarus_data_spec.Statistics.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.Statistics.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1006,
  serialized_end=1055,
)

_STATISTICS_NULL = _descriptor.Descriptor(
  name='Null',
  full_name='sarus_data_spec.Statistics.Null',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='size', full_name='sarus_data_spec.Statistics.Null.size', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multiplicity', full_name='sarus_data_spec.Statistics.Null.multiplicity', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1057,
  serialized_end=1099,
)

_STATISTICS_UNIT = _descriptor.Descriptor(
  name='Unit',
  full_name='sarus_data_spec.Statistics.Unit',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='size', full_name='sarus_data_spec.Statistics.Unit.size', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multiplicity', full_name='sarus_data_spec.Statistics.Unit.multiplicity', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1101,
  serialized_end=1143,
)

_STATISTICS_BOOLEAN = _descriptor.Descriptor(
  name='Boolean',
  full_name='sarus_data_spec.Statistics.Boolean',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='distribution', full_name='sarus_data_spec.Statistics.Boolean.distribution', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size', full_name='sarus_data_spec.Statistics.Boolean.size', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multiplicity', full_name='sarus_data_spec.Statistics.Boolean.multiplicity', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1145,
  serialized_end=1243,
)

_STATISTICS_ID = _descriptor.Descriptor(
  name='Id',
  full_name='sarus_data_spec.Statistics.Id',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='size', full_name='sarus_data_spec.Statistics.Id.size', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multiplicity', full_name='sarus_data_spec.Statistics.Id.multiplicity', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1245,
  serialized_end=1285,
)

_STATISTICS_INTEGER = _descriptor.Descriptor(
  name='Integer',
  full_name='sarus_data_spec.Statistics.Integer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='distribution', full_name='sarus_data_spec.Statistics.Integer.distribution', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size', full_name='sarus_data_spec.Statistics.Integer.size', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multiplicity', full_name='sarus_data_spec.Statistics.Integer.multiplicity', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1287,
  serialized_end=1385,
)

_STATISTICS_ENUM = _descriptor.Descriptor(
  name='Enum',
  full_name='sarus_data_spec.Statistics.Enum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='distribution', full_name='sarus_data_spec.Statistics.Enum.distribution', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size', full_name='sarus_data_spec.Statistics.Enum.size', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multiplicity', full_name='sarus_data_spec.Statistics.Enum.multiplicity', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1387,
  serialized_end=1482,
)

_STATISTICS_FLOAT = _descriptor.Descriptor(
  name='Float',
  full_name='sarus_data_spec.Statistics.Float',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='distribution', full_name='sarus_data_spec.Statistics.Float.distribution', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size', full_name='sarus_data_spec.Statistics.Float.size', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multiplicity', full_name='sarus_data_spec.Statistics.Float.multiplicity', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1484,
  serialized_end=1580,
)

_STATISTICS_TEXT = _descriptor.Descriptor(
  name='Text',
  full_name='sarus_data_spec.Statistics.Text',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='distribution', full_name='sarus_data_spec.Statistics.Text.distribution', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='example', full_name='sarus_data_spec.Statistics.Text.example', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size', full_name='sarus_data_spec.Statistics.Text.size', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multiplicity', full_name='sarus_data_spec.Statistics.Text.multiplicity', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1582,
  serialized_end=1694,
)

_STATISTICS_BYTES = _descriptor.Descriptor(
  name='Bytes',
  full_name='sarus_data_spec.Statistics.Bytes',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='size', full_name='sarus_data_spec.Statistics.Bytes.size', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multiplicity', full_name='sarus_data_spec.Statistics.Bytes.multiplicity', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1696,
  serialized_end=1739,
)

_STATISTICS_STRUCT_FIELD = _descriptor.Descriptor(
  name='Field',
  full_name='sarus_data_spec.Statistics.Struct.Field',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='sarus_data_spec.Statistics.Struct.Field.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='statistics', full_name='sarus_data_spec.Statistics.Struct.Field.statistics', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1860,
  serialized_end=1930,
)

_STATISTICS_STRUCT = _descriptor.Descriptor(
  name='Struct',
  full_name='sarus_data_spec.Statistics.Struct',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='fields', full_name='sarus_data_spec.Statistics.Struct.fields', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size', full_name='sarus_data_spec.Statistics.Struct.size', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='sarus_data_spec.Statistics.Struct.name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multiplicity', full_name='sarus_data_spec.Statistics.Struct.multiplicity', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_STATISTICS_STRUCT_FIELD, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1742,
  serialized_end=1930,
)

_STATISTICS_UNION_FIELD = _descriptor.Descriptor(
  name='Field',
  full_name='sarus_data_spec.Statistics.Union.Field',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='sarus_data_spec.Statistics.Union.Field.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='statistics', full_name='sarus_data_spec.Statistics.Union.Field.statistics', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1860,
  serialized_end=1930,
)

_STATISTICS_UNION = _descriptor.Descriptor(
  name='Union',
  full_name='sarus_data_spec.Statistics.Union',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='fields', full_name='sarus_data_spec.Statistics.Union.fields', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size', full_name='sarus_data_spec.Statistics.Union.size', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='sarus_data_spec.Statistics.Union.name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multiplicity', full_name='sarus_data_spec.Statistics.Union.multiplicity', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_STATISTICS_UNION_FIELD, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1933,
  serialized_end=2119,
)

_STATISTICS_OPTIONAL = _descriptor.Descriptor(
  name='Optional',
  full_name='sarus_data_spec.Statistics.Optional',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='statistics', full_name='sarus_data_spec.Statistics.Optional.statistics', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size', full_name='sarus_data_spec.Statistics.Optional.size', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='sarus_data_spec.Statistics.Optional.name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multiplicity', full_name='sarus_data_spec.Statistics.Optional.multiplicity', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2121,
  serialized_end=2230,
)

_STATISTICS_LIST = _descriptor.Descriptor(
  name='List',
  full_name='sarus_data_spec.Statistics.List',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='statistics', full_name='sarus_data_spec.Statistics.List.statistics', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='distribution', full_name='sarus_data_spec.Statistics.List.distribution', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size', full_name='sarus_data_spec.Statistics.List.size', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multiplicity', full_name='sarus_data_spec.Statistics.List.multiplicity', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2233,
  serialized_end=2377,
)

_STATISTICS_ARRAY = _descriptor.Descriptor(
  name='Array',
  full_name='sarus_data_spec.Statistics.Array',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='statistics', full_name='sarus_data_spec.Statistics.Array.statistics', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='distributions', full_name='sarus_data_spec.Statistics.Array.distributions', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size', full_name='sarus_data_spec.Statistics.Array.size', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multiplicity', full_name='sarus_data_spec.Statistics.Array.multiplicity', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2380,
  serialized_end=2526,
)

_STATISTICS_DATETIME = _descriptor.Descriptor(
  name='Datetime',
  full_name='sarus_data_spec.Statistics.Datetime',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='distribution', full_name='sarus_data_spec.Statistics.Datetime.distribution', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size', full_name='sarus_data_spec.Statistics.Datetime.size', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multiplicity', full_name='sarus_data_spec.Statistics.Datetime.multiplicity', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2528,
  serialized_end=2627,
)

_STATISTICS_CONSTRAINED = _descriptor.Descriptor(
  name='Constrained',
  full_name='sarus_data_spec.Statistics.Constrained',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='statistics', full_name='sarus_data_spec.Statistics.Constrained.statistics', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size', full_name='sarus_data_spec.Statistics.Constrained.size', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='multiplicity', full_name='sarus_data_spec.Statistics.Constrained.multiplicity', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2629,
  serialized_end=2727,
)

_STATISTICS = _descriptor.Descriptor(
  name='Statistics',
  full_name='sarus_data_spec.Statistics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='sarus_data_spec.Statistics.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='null', full_name='sarus_data_spec.Statistics.null', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unit', full_name='sarus_data_spec.Statistics.unit', index=2,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='boolean', full_name='sarus_data_spec.Statistics.boolean', index=3,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='integer', full_name='sarus_data_spec.Statistics.integer', index=4,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enum', full_name='sarus_data_spec.Statistics.enum', index=5,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='float', full_name='sarus_data_spec.Statistics.float', index=6,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='text', full_name='sarus_data_spec.Statistics.text', index=7,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bytes', full_name='sarus_data_spec.Statistics.bytes', index=8,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='struct', full_name='sarus_data_spec.Statistics.struct', index=9,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='union', full_name='sarus_data_spec.Statistics.union', index=10,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='optional', full_name='sarus_data_spec.Statistics.optional', index=11,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='list', full_name='sarus_data_spec.Statistics.list', index=12,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='array', full_name='sarus_data_spec.Statistics.array', index=13,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='datetime', full_name='sarus_data_spec.Statistics.datetime', index=14,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='id', full_name='sarus_data_spec.Statistics.id', index=15,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='constrained', full_name='sarus_data_spec.Statistics.constrained', index=16,
      number=18, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='properties', full_name='sarus_data_spec.Statistics.properties', index=17,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_STATISTICS_PROPERTIESENTRY, _STATISTICS_NULL, _STATISTICS_UNIT, _STATISTICS_BOOLEAN, _STATISTICS_ID, _STATISTICS_INTEGER, _STATISTICS_ENUM, _STATISTICS_FLOAT, _STATISTICS_TEXT, _STATISTICS_BYTES, _STATISTICS_STRUCT, _STATISTICS_UNION, _STATISTICS_OPTIONAL, _STATISTICS_LIST, _STATISTICS_ARRAY, _STATISTICS_DATETIME, _STATISTICS_CONSTRAINED, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='statistics', full_name='sarus_data_spec.Statistics.statistics',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=63,
  serialized_end=2741,
)


_DISTRIBUTION_PROPERTIESENTRY = _descriptor.Descriptor(
  name='PropertiesEntry',
  full_name='sarus_data_spec.Distribution.PropertiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='sarus_data_spec.Distribution.PropertiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.Distribution.PropertiesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1006,
  serialized_end=1055,
)

_DISTRIBUTION_DOUBLE_POINT = _descriptor.Descriptor(
  name='Point',
  full_name='sarus_data_spec.Distribution.Double.Point',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.Distribution.Double.Point.value', index=0,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='probability', full_name='sarus_data_spec.Distribution.Double.Point.probability', index=1,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3199,
  serialized_end=3242,
)

_DISTRIBUTION_DOUBLE = _descriptor.Descriptor(
  name='Double',
  full_name='sarus_data_spec.Distribution.Double',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='points', full_name='sarus_data_spec.Distribution.Double.points', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min', full_name='sarus_data_spec.Distribution.Double.min', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max', full_name='sarus_data_spec.Distribution.Double.max', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_DISTRIBUTION_DOUBLE_POINT, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3103,
  serialized_end=3242,
)

_DISTRIBUTION_INTEGER_POINT = _descriptor.Descriptor(
  name='Point',
  full_name='sarus_data_spec.Distribution.Integer.Point',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.Distribution.Integer.Point.value', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='probability', full_name='sarus_data_spec.Distribution.Integer.Point.probability', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3343,
  serialized_end=3386,
)

_DISTRIBUTION_INTEGER = _descriptor.Descriptor(
  name='Integer',
  full_name='sarus_data_spec.Distribution.Integer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='points', full_name='sarus_data_spec.Distribution.Integer.points', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min', full_name='sarus_data_spec.Distribution.Integer.min', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max', full_name='sarus_data_spec.Distribution.Integer.max', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_DISTRIBUTION_INTEGER_POINT, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3245,
  serialized_end=3386,
)

_DISTRIBUTION_BOOLEAN_POINT = _descriptor.Descriptor(
  name='Point',
  full_name='sarus_data_spec.Distribution.Boolean.Point',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='sarus_data_spec.Distribution.Boolean.Point.name', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.Distribution.Boolean.Point.value', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='probability', full_name='sarus_data_spec.Distribution.Boolean.Point.probability', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3461,
  serialized_end=3518,
)

_DISTRIBUTION_BOOLEAN = _descriptor.Descriptor(
  name='Boolean',
  full_name='sarus_data_spec.Distribution.Boolean',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='points', full_name='sarus_data_spec.Distribution.Boolean.points', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_DISTRIBUTION_BOOLEAN_POINT, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3389,
  serialized_end=3518,
)

_DISTRIBUTION_ENUM_POINT = _descriptor.Descriptor(
  name='Point',
  full_name='sarus_data_spec.Distribution.Enum.Point',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='sarus_data_spec.Distribution.Enum.Point.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='sarus_data_spec.Distribution.Enum.Point.value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='probability', full_name='sarus_data_spec.Distribution.Enum.Point.probability', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3586,
  serialized_end=3643,
)

_DISTRIBUTION_ENUM = _descriptor.Descriptor(
  name='Enum',
  full_name='sarus_data_spec.Distribution.Enum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='points', full_name='sarus_data_spec.Distribution.Enum.points', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_DISTRIBUTION_ENUM_POINT, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3520,
  serialized_end=3643,
)

_DISTRIBUTION = _descriptor.Descriptor(
  name='Distribution',
  full_name='sarus_data_spec.Distribution',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='double', full_name='sarus_data_spec.Distribution.double', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='integer', full_name='sarus_data_spec.Distribution.integer', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='boolean', full_name='sarus_data_spec.Distribution.boolean', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enum', full_name='sarus_data_spec.Distribution.enum', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='properties', full_name='sarus_data_spec.Distribution.properties', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_DISTRIBUTION_PROPERTIESENTRY, _DISTRIBUTION_DOUBLE, _DISTRIBUTION_INTEGER, _DISTRIBUTION_BOOLEAN, _DISTRIBUTION_ENUM, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='distribution', full_name='sarus_data_spec.Distribution.distribution',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=2744,
  serialized_end=3659,
)

_STATISTICS_PROPERTIESENTRY.containing_type = _STATISTICS
_STATISTICS_NULL.containing_type = _STATISTICS
_STATISTICS_UNIT.containing_type = _STATISTICS
_STATISTICS_BOOLEAN.fields_by_name['distribution'].message_type = _DISTRIBUTION
_STATISTICS_BOOLEAN.containing_type = _STATISTICS
_STATISTICS_ID.containing_type = _STATISTICS
_STATISTICS_INTEGER.fields_by_name['distribution'].message_type = _DISTRIBUTION
_STATISTICS_INTEGER.containing_type = _STATISTICS
_STATISTICS_ENUM.fields_by_name['distribution'].message_type = _DISTRIBUTION
_STATISTICS_ENUM.containing_type = _STATISTICS
_STATISTICS_FLOAT.fields_by_name['distribution'].message_type = _DISTRIBUTION
_STATISTICS_FLOAT.containing_type = _STATISTICS
_STATISTICS_TEXT.fields_by_name['distribution'].message_type = _DISTRIBUTION
_STATISTICS_TEXT.containing_type = _STATISTICS
_STATISTICS_BYTES.containing_type = _STATISTICS
_STATISTICS_STRUCT_FIELD.fields_by_name['statistics'].message_type = _STATISTICS
_STATISTICS_STRUCT_FIELD.containing_type = _STATISTICS_STRUCT
_STATISTICS_STRUCT.fields_by_name['fields'].message_type = _STATISTICS_STRUCT_FIELD
_STATISTICS_STRUCT.containing_type = _STATISTICS
_STATISTICS_UNION_FIELD.fields_by_name['statistics'].message_type = _STATISTICS
_STATISTICS_UNION_FIELD.containing_type = _STATISTICS_UNION
_STATISTICS_UNION.fields_by_name['fields'].message_type = _STATISTICS_UNION_FIELD
_STATISTICS_UNION.containing_type = _STATISTICS
_STATISTICS_OPTIONAL.fields_by_name['statistics'].message_type = _STATISTICS
_STATISTICS_OPTIONAL.containing_type = _STATISTICS
_STATISTICS_LIST.fields_by_name['statistics'].message_type = _STATISTICS
_STATISTICS_LIST.fields_by_name['distribution'].message_type = _DISTRIBUTION
_STATISTICS_LIST.containing_type = _STATISTICS
_STATISTICS_ARRAY.fields_by_name['statistics'].message_type = _STATISTICS
_STATISTICS_ARRAY.fields_by_name['distributions'].message_type = _DISTRIBUTION
_STATISTICS_ARRAY.containing_type = _STATISTICS
_STATISTICS_DATETIME.fields_by_name['distribution'].message_type = _DISTRIBUTION
_STATISTICS_DATETIME.containing_type = _STATISTICS
_STATISTICS_CONSTRAINED.fields_by_name['statistics'].message_type = _STATISTICS
_STATISTICS_CONSTRAINED.containing_type = _STATISTICS
_STATISTICS.fields_by_name['null'].message_type = _STATISTICS_NULL
_STATISTICS.fields_by_name['unit'].message_type = _STATISTICS_UNIT
_STATISTICS.fields_by_name['boolean'].message_type = _STATISTICS_BOOLEAN
_STATISTICS.fields_by_name['integer'].message_type = _STATISTICS_INTEGER
_STATISTICS.fields_by_name['enum'].message_type = _STATISTICS_ENUM
_STATISTICS.fields_by_name['float'].message_type = _STATISTICS_FLOAT
_STATISTICS.fields_by_name['text'].message_type = _STATISTICS_TEXT
_STATISTICS.fields_by_name['bytes'].message_type = _STATISTICS_BYTES
_STATISTICS.fields_by_name['struct'].message_type = _STATISTICS_STRUCT
_STATISTICS.fields_by_name['union'].message_type = _STATISTICS_UNION
_STATISTICS.fields_by_name['optional'].message_type = _STATISTICS_OPTIONAL
_STATISTICS.fields_by_name['list'].message_type = _STATISTICS_LIST
_STATISTICS.fields_by_name['array'].message_type = _STATISTICS_ARRAY
_STATISTICS.fields_by_name['datetime'].message_type = _STATISTICS_DATETIME
_STATISTICS.fields_by_name['id'].message_type = _STATISTICS_ID
_STATISTICS.fields_by_name['constrained'].message_type = _STATISTICS_CONSTRAINED
_STATISTICS.fields_by_name['properties'].message_type = _STATISTICS_PROPERTIESENTRY
_STATISTICS.oneofs_by_name['statistics'].fields.append(
  _STATISTICS.fields_by_name['null'])
_STATISTICS.fields_by_name['null'].containing_oneof = _STATISTICS.oneofs_by_name['statistics']
_STATISTICS.oneofs_by_name['statistics'].fields.append(
  _STATISTICS.fields_by_name['unit'])
_STATISTICS.fields_by_name['unit'].containing_oneof = _STATISTICS.oneofs_by_name['statistics']
_STATISTICS.oneofs_by_name['statistics'].fields.append(
  _STATISTICS.fields_by_name['boolean'])
_STATISTICS.fields_by_name['boolean'].containing_oneof = _STATISTICS.oneofs_by_name['statistics']
_STATISTICS.oneofs_by_name['statistics'].fields.append(
  _STATISTICS.fields_by_name['integer'])
_STATISTICS.fields_by_name['integer'].containing_oneof = _STATISTICS.oneofs_by_name['statistics']
_STATISTICS.oneofs_by_name['statistics'].fields.append(
  _STATISTICS.fields_by_name['enum'])
_STATISTICS.fields_by_name['enum'].containing_oneof = _STATISTICS.oneofs_by_name['statistics']
_STATISTICS.oneofs_by_name['statistics'].fields.append(
  _STATISTICS.fields_by_name['float'])
_STATISTICS.fields_by_name['float'].containing_oneof = _STATISTICS.oneofs_by_name['statistics']
_STATISTICS.oneofs_by_name['statistics'].fields.append(
  _STATISTICS.fields_by_name['text'])
_STATISTICS.fields_by_name['text'].containing_oneof = _STATISTICS.oneofs_by_name['statistics']
_STATISTICS.oneofs_by_name['statistics'].fields.append(
  _STATISTICS.fields_by_name['bytes'])
_STATISTICS.fields_by_name['bytes'].containing_oneof = _STATISTICS.oneofs_by_name['statistics']
_STATISTICS.oneofs_by_name['statistics'].fields.append(
  _STATISTICS.fields_by_name['struct'])
_STATISTICS.fields_by_name['struct'].containing_oneof = _STATISTICS.oneofs_by_name['statistics']
_STATISTICS.oneofs_by_name['statistics'].fields.append(
  _STATISTICS.fields_by_name['union'])
_STATISTICS.fields_by_name['union'].containing_oneof = _STATISTICS.oneofs_by_name['statistics']
_STATISTICS.oneofs_by_name['statistics'].fields.append(
  _STATISTICS.fields_by_name['optional'])
_STATISTICS.fields_by_name['optional'].containing_oneof = _STATISTICS.oneofs_by_name['statistics']
_STATISTICS.oneofs_by_name['statistics'].fields.append(
  _STATISTICS.fields_by_name['list'])
_STATISTICS.fields_by_name['list'].containing_oneof = _STATISTICS.oneofs_by_name['statistics']
_STATISTICS.oneofs_by_name['statistics'].fields.append(
  _STATISTICS.fields_by_name['array'])
_STATISTICS.fields_by_name['array'].containing_oneof = _STATISTICS.oneofs_by_name['statistics']
_STATISTICS.oneofs_by_name['statistics'].fields.append(
  _STATISTICS.fields_by_name['datetime'])
_STATISTICS.fields_by_name['datetime'].containing_oneof = _STATISTICS.oneofs_by_name['statistics']
_STATISTICS.oneofs_by_name['statistics'].fields.append(
  _STATISTICS.fields_by_name['id'])
_STATISTICS.fields_by_name['id'].containing_oneof = _STATISTICS.oneofs_by_name['statistics']
_STATISTICS.oneofs_by_name['statistics'].fields.append(
  _STATISTICS.fields_by_name['constrained'])
_STATISTICS.fields_by_name['constrained'].containing_oneof = _STATISTICS.oneofs_by_name['statistics']
_DISTRIBUTION_PROPERTIESENTRY.containing_type = _DISTRIBUTION
_DISTRIBUTION_DOUBLE_POINT.containing_type = _DISTRIBUTION_DOUBLE
_DISTRIBUTION_DOUBLE.fields_by_name['points'].message_type = _DISTRIBUTION_DOUBLE_POINT
_DISTRIBUTION_DOUBLE.containing_type = _DISTRIBUTION
_DISTRIBUTION_INTEGER_POINT.containing_type = _DISTRIBUTION_INTEGER
_DISTRIBUTION_INTEGER.fields_by_name['points'].message_type = _DISTRIBUTION_INTEGER_POINT
_DISTRIBUTION_INTEGER.containing_type = _DISTRIBUTION
_DISTRIBUTION_BOOLEAN_POINT.containing_type = _DISTRIBUTION_BOOLEAN
_DISTRIBUTION_BOOLEAN.fields_by_name['points'].message_type = _DISTRIBUTION_BOOLEAN_POINT
_DISTRIBUTION_BOOLEAN.containing_type = _DISTRIBUTION
_DISTRIBUTION_ENUM_POINT.containing_type = _DISTRIBUTION_ENUM
_DISTRIBUTION_ENUM.fields_by_name['points'].message_type = _DISTRIBUTION_ENUM_POINT
_DISTRIBUTION_ENUM.containing_type = _DISTRIBUTION
_DISTRIBUTION.fields_by_name['double'].message_type = _DISTRIBUTION_DOUBLE
_DISTRIBUTION.fields_by_name['integer'].message_type = _DISTRIBUTION_INTEGER
_DISTRIBUTION.fields_by_name['boolean'].message_type = _DISTRIBUTION_BOOLEAN
_DISTRIBUTION.fields_by_name['enum'].message_type = _DISTRIBUTION_ENUM
_DISTRIBUTION.fields_by_name['properties'].message_type = _DISTRIBUTION_PROPERTIESENTRY
_DISTRIBUTION.oneofs_by_name['distribution'].fields.append(
  _DISTRIBUTION.fields_by_name['double'])
_DISTRIBUTION.fields_by_name['double'].containing_oneof = _DISTRIBUTION.oneofs_by_name['distribution']
_DISTRIBUTION.oneofs_by_name['distribution'].fields.append(
  _DISTRIBUTION.fields_by_name['integer'])
_DISTRIBUTION.fields_by_name['integer'].containing_oneof = _DISTRIBUTION.oneofs_by_name['distribution']
_DISTRIBUTION.oneofs_by_name['distribution'].fields.append(
  _DISTRIBUTION.fields_by_name['boolean'])
_DISTRIBUTION.fields_by_name['boolean'].containing_oneof = _DISTRIBUTION.oneofs_by_name['distribution']
_DISTRIBUTION.oneofs_by_name['distribution'].fields.append(
  _DISTRIBUTION.fields_by_name['enum'])
_DISTRIBUTION.fields_by_name['enum'].containing_oneof = _DISTRIBUTION.oneofs_by_name['distribution']
DESCRIPTOR.message_types_by_name['Statistics'] = _STATISTICS
DESCRIPTOR.message_types_by_name['Distribution'] = _DISTRIBUTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Statistics = _reflection.GeneratedProtocolMessageType('Statistics', (_message.Message,), {

  'PropertiesEntry' : _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), {
    'DESCRIPTOR' : _STATISTICS_PROPERTIESENTRY,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.PropertiesEntry)
    })
  ,

  'Null' : _reflection.GeneratedProtocolMessageType('Null', (_message.Message,), {
    'DESCRIPTOR' : _STATISTICS_NULL,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Null)
    })
  ,

  'Unit' : _reflection.GeneratedProtocolMessageType('Unit', (_message.Message,), {
    'DESCRIPTOR' : _STATISTICS_UNIT,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Unit)
    })
  ,

  'Boolean' : _reflection.GeneratedProtocolMessageType('Boolean', (_message.Message,), {
    'DESCRIPTOR' : _STATISTICS_BOOLEAN,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Boolean)
    })
  ,

  'Id' : _reflection.GeneratedProtocolMessageType('Id', (_message.Message,), {
    'DESCRIPTOR' : _STATISTICS_ID,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Id)
    })
  ,

  'Integer' : _reflection.GeneratedProtocolMessageType('Integer', (_message.Message,), {
    'DESCRIPTOR' : _STATISTICS_INTEGER,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Integer)
    })
  ,

  'Enum' : _reflection.GeneratedProtocolMessageType('Enum', (_message.Message,), {
    'DESCRIPTOR' : _STATISTICS_ENUM,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Enum)
    })
  ,

  'Float' : _reflection.GeneratedProtocolMessageType('Float', (_message.Message,), {
    'DESCRIPTOR' : _STATISTICS_FLOAT,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Float)
    })
  ,

  'Text' : _reflection.GeneratedProtocolMessageType('Text', (_message.Message,), {
    'DESCRIPTOR' : _STATISTICS_TEXT,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Text)
    })
  ,

  'Bytes' : _reflection.GeneratedProtocolMessageType('Bytes', (_message.Message,), {
    'DESCRIPTOR' : _STATISTICS_BYTES,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Bytes)
    })
  ,

  'Struct' : _reflection.GeneratedProtocolMessageType('Struct', (_message.Message,), {

    'Field' : _reflection.GeneratedProtocolMessageType('Field', (_message.Message,), {
      'DESCRIPTOR' : _STATISTICS_STRUCT_FIELD,
      '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
      # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Struct.Field)
      })
    ,
    'DESCRIPTOR' : _STATISTICS_STRUCT,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Struct)
    })
  ,

  'Union' : _reflection.GeneratedProtocolMessageType('Union', (_message.Message,), {

    'Field' : _reflection.GeneratedProtocolMessageType('Field', (_message.Message,), {
      'DESCRIPTOR' : _STATISTICS_UNION_FIELD,
      '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
      # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Union.Field)
      })
    ,
    'DESCRIPTOR' : _STATISTICS_UNION,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Union)
    })
  ,

  'Optional' : _reflection.GeneratedProtocolMessageType('Optional', (_message.Message,), {
    'DESCRIPTOR' : _STATISTICS_OPTIONAL,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Optional)
    })
  ,

  'List' : _reflection.GeneratedProtocolMessageType('List', (_message.Message,), {
    'DESCRIPTOR' : _STATISTICS_LIST,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.List)
    })
  ,

  'Array' : _reflection.GeneratedProtocolMessageType('Array', (_message.Message,), {
    'DESCRIPTOR' : _STATISTICS_ARRAY,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Array)
    })
  ,

  'Datetime' : _reflection.GeneratedProtocolMessageType('Datetime', (_message.Message,), {
    'DESCRIPTOR' : _STATISTICS_DATETIME,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Datetime)
    })
  ,

  'Constrained' : _reflection.GeneratedProtocolMessageType('Constrained', (_message.Message,), {
    'DESCRIPTOR' : _STATISTICS_CONSTRAINED,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics.Constrained)
    })
  ,
  'DESCRIPTOR' : _STATISTICS,
  '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
  # @@protoc_insertion_point(class_scope:sarus_data_spec.Statistics)
  })
_sym_db.RegisterMessage(Statistics)
_sym_db.RegisterMessage(Statistics.PropertiesEntry)
_sym_db.RegisterMessage(Statistics.Null)
_sym_db.RegisterMessage(Statistics.Unit)
_sym_db.RegisterMessage(Statistics.Boolean)
_sym_db.RegisterMessage(Statistics.Id)
_sym_db.RegisterMessage(Statistics.Integer)
_sym_db.RegisterMessage(Statistics.Enum)
_sym_db.RegisterMessage(Statistics.Float)
_sym_db.RegisterMessage(Statistics.Text)
_sym_db.RegisterMessage(Statistics.Bytes)
_sym_db.RegisterMessage(Statistics.Struct)
_sym_db.RegisterMessage(Statistics.Struct.Field)
_sym_db.RegisterMessage(Statistics.Union)
_sym_db.RegisterMessage(Statistics.Union.Field)
_sym_db.RegisterMessage(Statistics.Optional)
_sym_db.RegisterMessage(Statistics.List)
_sym_db.RegisterMessage(Statistics.Array)
_sym_db.RegisterMessage(Statistics.Datetime)
_sym_db.RegisterMessage(Statistics.Constrained)

Distribution = _reflection.GeneratedProtocolMessageType('Distribution', (_message.Message,), {

  'PropertiesEntry' : _reflection.GeneratedProtocolMessageType('PropertiesEntry', (_message.Message,), {
    'DESCRIPTOR' : _DISTRIBUTION_PROPERTIESENTRY,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Distribution.PropertiesEntry)
    })
  ,

  'Double' : _reflection.GeneratedProtocolMessageType('Double', (_message.Message,), {

    'Point' : _reflection.GeneratedProtocolMessageType('Point', (_message.Message,), {
      'DESCRIPTOR' : _DISTRIBUTION_DOUBLE_POINT,
      '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
      # @@protoc_insertion_point(class_scope:sarus_data_spec.Distribution.Double.Point)
      })
    ,
    'DESCRIPTOR' : _DISTRIBUTION_DOUBLE,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Distribution.Double)
    })
  ,

  'Integer' : _reflection.GeneratedProtocolMessageType('Integer', (_message.Message,), {

    'Point' : _reflection.GeneratedProtocolMessageType('Point', (_message.Message,), {
      'DESCRIPTOR' : _DISTRIBUTION_INTEGER_POINT,
      '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
      # @@protoc_insertion_point(class_scope:sarus_data_spec.Distribution.Integer.Point)
      })
    ,
    'DESCRIPTOR' : _DISTRIBUTION_INTEGER,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Distribution.Integer)
    })
  ,

  'Boolean' : _reflection.GeneratedProtocolMessageType('Boolean', (_message.Message,), {

    'Point' : _reflection.GeneratedProtocolMessageType('Point', (_message.Message,), {
      'DESCRIPTOR' : _DISTRIBUTION_BOOLEAN_POINT,
      '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
      # @@protoc_insertion_point(class_scope:sarus_data_spec.Distribution.Boolean.Point)
      })
    ,
    'DESCRIPTOR' : _DISTRIBUTION_BOOLEAN,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Distribution.Boolean)
    })
  ,

  'Enum' : _reflection.GeneratedProtocolMessageType('Enum', (_message.Message,), {

    'Point' : _reflection.GeneratedProtocolMessageType('Point', (_message.Message,), {
      'DESCRIPTOR' : _DISTRIBUTION_ENUM_POINT,
      '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
      # @@protoc_insertion_point(class_scope:sarus_data_spec.Distribution.Enum.Point)
      })
    ,
    'DESCRIPTOR' : _DISTRIBUTION_ENUM,
    '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
    # @@protoc_insertion_point(class_scope:sarus_data_spec.Distribution.Enum)
    })
  ,
  'DESCRIPTOR' : _DISTRIBUTION,
  '__module__' : 'sarus_data_spec.protobuf.statistics_pb2'
  # @@protoc_insertion_point(class_scope:sarus_data_spec.Distribution)
  })
_sym_db.RegisterMessage(Distribution)
_sym_db.RegisterMessage(Distribution.PropertiesEntry)
_sym_db.RegisterMessage(Distribution.Double)
_sym_db.RegisterMessage(Distribution.Double.Point)
_sym_db.RegisterMessage(Distribution.Integer)
_sym_db.RegisterMessage(Distribution.Integer.Point)
_sym_db.RegisterMessage(Distribution.Boolean)
_sym_db.RegisterMessage(Distribution.Boolean.Point)
_sym_db.RegisterMessage(Distribution.Enum)
_sym_db.RegisterMessage(Distribution.Enum.Point)


_STATISTICS_PROPERTIESENTRY._options = None
_DISTRIBUTION_PROPERTIESENTRY._options = None
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

from functools import reduce
from typing import Collection, Dict, List, Optional, cast

from sarus_data_spec.storage.typing import Storage
import sarus_data_spec.protobuf as sp
import sarus_data_spec.query_manager.simple_rules as compilation_rules
import sarus_data_spec.typing as st


class BaseQueryManager:
    def __init__(self, storage: Storage):
        self._storage = storage

    def storage(self) -> Storage:
        return self._storage

    def is_compliant(
        self,
        dataspec: st.DataSpec,
        kind: st.ConstraintKind,
        public_context: List[str],
        epsilon: Optional[float],
    ) -> bool:
        variant = self.variant(
            dataspec,
            kind,
            public_context,
            epsilon,
        )
        if variant:
            return variant.uuid() == dataspec.uuid()
        else:
            return False

    def variant(
        self,
        dataspec: st.DataSpec,
        kind: st.ConstraintKind,
        public_context: List[str],
        epsilon: Optional[float],
    ) -> Optional[st.DataSpec]:
        return compilation_rules.compile(
            self, dataspec, kind, public_context, epsilon
        )

    def variants(self, dataspec: st.DataSpec) -> Collection[st.DataSpec]:
        attributes = self.storage().referring(
            dataspec, type_name=sp.type_name(sp.Attribute)
        )
        all_attributes: Dict[str, str] = reduce(
            lambda x, y: {**x, **y},
            map(lambda x: x.properties(), attributes),
            dict(),
        )
        variants_uuids = [
            uuid
            for key, uuid in all_attributes.items()
            if key in sp.ConstraintKind.DESCRIPTOR.values_by_name
        ]
        variants = [
            cast(st.DataSpec, self.storage().referrable(uuid))
            for uuid in variants_uuids
        ]
        return [var for var in variants if var]

    def variant_constraint(
        self, dataspec: st.DataSpec
    ) -> Optional[st.VariantConstraint]:
        constraints = self.storage().referring(
            dataspec, type_name=sp.type_name(sp.VariantConstraint)
        )
        if len(constraints) == 0:
            return None
        elif len(constraints) == 1:
            return cast(st.VariantConstraint, list(constraints)[0])
        else:
            raise ValueError(
                f"More than one variant_constraint attached to {dataspec}"
            )

    def verifies(
        self,
        variant_constraint: st.VariantConstraint,
        kind: st.ConstraintKind,
        public_context: Collection[str],
        epsilon: Optional[float],
    ) -> bool:
        return compilation_rules.verifies(
            query_manager=self,
            variant_constraint=variant_constraint,
            kind=kind,
            public_context=public_context,
            epsilon=epsilon,
        )

    def is_pe_preserving(self, transform: st.Transform) -> bool:
        raise NotImplementedError("is_pe_preserving")

    def is_differentially_private(self, transform: st.Transform) -> bool:
        raise NotImplementedError("is_differentially_private")

    def transform_equivalent(
        self, transform: st.Transform, dp: bool
    ) -> Optional[st.Transform]:
        """Return the DP or non-DP version of a transform.

        Return None if the requested DP equivalent does not exist.
        """
        raise NotImplementedError("transform_equivalent")



from typing import AsyncIterator, List, Union, cast

import numpy as np
import pyarrow as pa

from sarus_data_spec.manager.asyncio.utils import async_iter
import sarus_data_spec.protobuf as sp
import sarus_data_spec.typing as st


async def arrow_sample(
    dataset: st.Dataset, batch_size: int
) -> AsyncIterator[pa.RecordBatch]:
    """Samples batches from parent dataset"""
    args, kwargs = dataset.parents()
    parents = args + list(kwargs.values())
    assert len(parents) == 1
    (parent,) = parents
    assert parent.prototype() == sp.Dataset
    ds_parent = cast(st.Dataset, parent)
    async_parent_batches = await ds_parent.async_to_arrow()
    parent_batches = [batch async for batch in async_parent_batches]
    parent_table = pa.Table.from_batches(parent_batches)

    if dataset.transform().protobuf().spec.sample.HasField('fraction'):
        indices = np.random.choice(
            parent_table.num_rows,
            replace=False,
            size=int(
                dataset.transform().protobuf().spec.sample.fraction
                * parent_table.num_rows
            ),
        )
    else:
        indices = np.random.choice(
            parent_table.num_rows,
            replace=False,
            size=dataset.transform().protobuf().spec.sample.size,
        )
    return fast_gather(
        indices=indices,
        batches=parent_table.to_batches(max_chunksize=1000),
        batch_size=batch_size,
    )


# Copied from
# https://github.com/huggingface/datasets/blob/master/src/datasets/table.py
def fast_gather(
    indices: Union[List[int], np.ndarray],
    batches: List[pa.RecordBatch],
    batch_size: int,
) -> AsyncIterator[pa.RecordBatch]:
    """
    Create a pa.Table by gathering the records at the records at the specified
    indices. Should be faster than
    pa.concat_tables(
        table.fast_slice(int(i) % table.num_rows, 1) for i in indices
    )
    since NumPy can compute the binary searches in parallel,
    highly optimized C
    """
    assert len(indices), "Indices must be non-empty"
    offsets: np.ndarray = np.cumsum(
        [0] + [len(b) for b in batches], dtype=np.int64
    )
    batch_indices = np.searchsorted(offsets, indices, side="right") - 1
    return async_iter(
        pa.Table.from_batches(
            [
                batches[batch_idx].slice(i - offsets[batch_idx], 1)
                for batch_idx, i in zip(batch_indices, indices)
            ],
            schema=batches[0].schema,
        ).to_batches(max_chunksize=batch_size)
    )



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class Scalar(google.protobuf.message.Message):
    """A Scalar represents data that does not fulfill the promise of a Dataset.
    A Dataset promises to have a schema and the possibility to iterate on
    pyarrow.RecordBatches
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    class PropertiesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: typing.Text = ...
        value: typing.Text = ...
        def __init__(self,
            *,
            key : typing.Text = ...,
            value : typing.Text = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"key",b"key",u"value",b"value"]) -> None: ...

    class Spec(google.protobuf.message.Message):
        """Definitions
        How to obtain the dataset
        """
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        TRANSFORMED_FIELD_NUMBER: builtins.int
        MODEL_FIELD_NUMBER: builtins.int
        @property
        def transformed(self) -> global___Scalar.Transformed: ...
        @property
        def model(self) -> global___Scalar.Model: ...
        def __init__(self,
            *,
            transformed : typing.Optional[global___Scalar.Transformed] = ...,
            model : typing.Optional[global___Scalar.Model] = ...,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal[u"model",b"model",u"spec",b"spec",u"transformed",b"transformed"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"model",b"model",u"spec",b"spec",u"transformed",b"transformed"]) -> None: ...
        def WhichOneof(self, oneof_group: typing_extensions.Literal[u"spec",b"spec"]) -> typing.Optional[typing_extensions.Literal["transformed","model"]]: ...

    class Transformed(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        class NamedArgumentsEntry(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
            KEY_FIELD_NUMBER: builtins.int
            VALUE_FIELD_NUMBER: builtins.int
            key: typing.Text = ...
            value: typing.Text = ...
            def __init__(self,
                *,
                key : typing.Text = ...,
                value : typing.Text = ...,
                ) -> None: ...
            def ClearField(self, field_name: typing_extensions.Literal[u"key",b"key",u"value",b"value"]) -> None: ...

        TRANSFORM_FIELD_NUMBER: builtins.int
        ARGUMENTS_FIELD_NUMBER: builtins.int
        NAMED_ARGUMENTS_FIELD_NUMBER: builtins.int
        transform: typing.Text = ...
        """Transform id"""

        @property
        def arguments(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]:
            """Dataset or other object ids"""
            pass
        @property
        def named_arguments(self) -> google.protobuf.internal.containers.ScalarMap[typing.Text, typing.Text]: ...
        def __init__(self,
            *,
            transform : typing.Text = ...,
            arguments : typing.Optional[typing.Iterable[typing.Text]] = ...,
            named_arguments : typing.Optional[typing.Mapping[typing.Text, typing.Text]] = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"arguments",b"arguments",u"named_arguments",b"named_arguments",u"transform",b"transform"]) -> None: ...

    class Model(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
        class ModelClass(_ModelClass, metaclass=_ModelClassEnumTypeWrapper):
            pass
        class _ModelClass:
            V = typing.NewType('V', builtins.int)
        class _ModelClassEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ModelClass.V], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
            TF_KERAS = Scalar.Model.ModelClass.V(0)
            SK_SVC = Scalar.Model.ModelClass.V(1)
            SK_ONEHOT = Scalar.Model.ModelClass.V(2)
            SK_PCA = Scalar.Model.ModelClass.V(3)
            SK_AFFINITY_PROPAGATION = Scalar.Model.ModelClass.V(4)
            """Cluster"""

            SK_AGGLOMERATIVE_CLUSTERING = Scalar.Model.ModelClass.V(5)
            SK_BIRCH = Scalar.Model.ModelClass.V(6)
            SK_DBSCAN = Scalar.Model.ModelClass.V(7)
            SK_FEATURE_AGGLOMERATION = Scalar.Model.ModelClass.V(8)
            SK_KMEANS = Scalar.Model.ModelClass.V(9)
            SK_MINIBATCH_KMEANS = Scalar.Model.ModelClass.V(10)
            SK_MEAN_SHIFT = Scalar.Model.ModelClass.V(11)
            SK_OPTICS = Scalar.Model.ModelClass.V(12)
            SK_SPECTRAL_CLUSTERING = Scalar.Model.ModelClass.V(13)
            SK_SPECTRAL_BICLUSTERING = Scalar.Model.ModelClass.V(14)
            SK_SPECTRAL_COCLUSTERING = Scalar.Model.ModelClass.V(15)
            SK_ADABOOST_CLASSIFIER = Scalar.Model.ModelClass.V(60)
            """ensemble"""

            SK_ADABOOST_REGRESSOR = Scalar.Model.ModelClass.V(61)
            SK_BAGGING_CLASSIFIER = Scalar.Model.ModelClass.V(62)
            SK_BAGGING_REGRESSOR = Scalar.Model.ModelClass.V(63)
            SK_EXTRA_TREES_REGRESSOR = Scalar.Model.ModelClass.V(64)
            SK_EXTRA_TREES_CLASSIFIER = Scalar.Model.ModelClass.V(65)
            SK_GRADIENT_BOOSTING_CLASSIFIER = Scalar.Model.ModelClass.V(66)
            SK_GRADIENT_BOOSTING_REGRESSOR = Scalar.Model.ModelClass.V(67)
            SK_ISOLATION_FOREST = Scalar.Model.ModelClass.V(68)
            SK_RANDOM_FOREST_CLASSIFIER = Scalar.Model.ModelClass.V(69)
            SK_RANDOM_FOREST_REGRESSOR = Scalar.Model.ModelClass.V(70)
            SK_RANDOM_TREES_EMBEDDING = Scalar.Model.ModelClass.V(71)
            SK_STACKING_CLASSIFIER = Scalar.Model.ModelClass.V(72)
            SK_STACKING_REGRESSOR = Scalar.Model.ModelClass.V(73)
            SK_VOTING_CLASSIFIER = Scalar.Model.ModelClass.V(74)
            SK_VOTING_REGRESSOR = Scalar.Model.ModelClass.V(75)
            SK_HIST_GRADIENT_BOOSTING_REGRESSOR = Scalar.Model.ModelClass.V(76)
            SK_HIST_GRADIENT_BOOSTING_CLASSIFIER = Scalar.Model.ModelClass.V(77)
            SK_REPEATED_STRATIFIED_KFOLD = Scalar.Model.ModelClass.V(80)
            """Model selection"""

            XGB_CLASSIFIER = Scalar.Model.ModelClass.V(92)
            """XGB"""

            SK_LABEL_ENCODER = Scalar.Model.ModelClass.V(98)

        TF_KERAS = Scalar.Model.ModelClass.V(0)
        SK_SVC = Scalar.Model.ModelClass.V(1)
        SK_ONEHOT = Scalar.Model.ModelClass.V(2)
        SK_PCA = Scalar.Model.ModelClass.V(3)
        SK_AFFINITY_PROPAGATION = Scalar.Model.ModelClass.V(4)
        """Cluster"""

        SK_AGGLOMERATIVE_CLUSTERING = Scalar.Model.ModelClass.V(5)
        SK_BIRCH = Scalar.Model.ModelClass.V(6)
        SK_DBSCAN = Scalar.Model.ModelClass.V(7)
        SK_FEATURE_AGGLOMERATION = Scalar.Model.ModelClass.V(8)
        SK_KMEANS = Scalar.Model.ModelClass.V(9)
        SK_MINIBATCH_KMEANS = Scalar.Model.ModelClass.V(10)
        SK_MEAN_SHIFT = Scalar.Model.ModelClass.V(11)
        SK_OPTICS = Scalar.Model.ModelClass.V(12)
        SK_SPECTRAL_CLUSTERING = Scalar.Model.ModelClass.V(13)
        SK_SPECTRAL_BICLUSTERING = Scalar.Model.ModelClass.V(14)
        SK_SPECTRAL_COCLUSTERING = Scalar.Model.ModelClass.V(15)
        SK_ADABOOST_CLASSIFIER = Scalar.Model.ModelClass.V(60)
        """ensemble"""

        SK_ADABOOST_REGRESSOR = Scalar.Model.ModelClass.V(61)
        SK_BAGGING_CLASSIFIER = Scalar.Model.ModelClass.V(62)
        SK_BAGGING_REGRESSOR = Scalar.Model.ModelClass.V(63)
        SK_EXTRA_TREES_REGRESSOR = Scalar.Model.ModelClass.V(64)
        SK_EXTRA_TREES_CLASSIFIER = Scalar.Model.ModelClass.V(65)
        SK_GRADIENT_BOOSTING_CLASSIFIER = Scalar.Model.ModelClass.V(66)
        SK_GRADIENT_BOOSTING_REGRESSOR = Scalar.Model.ModelClass.V(67)
        SK_ISOLATION_FOREST = Scalar.Model.ModelClass.V(68)
        SK_RANDOM_FOREST_CLASSIFIER = Scalar.Model.ModelClass.V(69)
        SK_RANDOM_FOREST_REGRESSOR = Scalar.Model.ModelClass.V(70)
        SK_RANDOM_TREES_EMBEDDING = Scalar.Model.ModelClass.V(71)
        SK_STACKING_CLASSIFIER = Scalar.Model.ModelClass.V(72)
        SK_STACKING_REGRESSOR = Scalar.Model.ModelClass.V(73)
        SK_VOTING_CLASSIFIER = Scalar.Model.ModelClass.V(74)
        SK_VOTING_REGRESSOR = Scalar.Model.ModelClass.V(75)
        SK_HIST_GRADIENT_BOOSTING_REGRESSOR = Scalar.Model.ModelClass.V(76)
        SK_HIST_GRADIENT_BOOSTING_CLASSIFIER = Scalar.Model.ModelClass.V(77)
        SK_REPEATED_STRATIFIED_KFOLD = Scalar.Model.ModelClass.V(80)
        """Model selection"""

        XGB_CLASSIFIER = Scalar.Model.ModelClass.V(92)
        """XGB"""

        SK_LABEL_ENCODER = Scalar.Model.ModelClass.V(98)

        ARGUMENTS_FIELD_NUMBER: builtins.int
        NAMED_ARGUMENTS_FIELD_NUMBER: builtins.int
        MODEL_CLASS_FIELD_NUMBER: builtins.int
        arguments: builtins.bytes = ...
        named_arguments: builtins.bytes = ...
        model_class: global___Scalar.Model.ModelClass.V = ...
        def __init__(self,
            *,
            arguments : builtins.bytes = ...,
            named_arguments : builtins.bytes = ...,
            model_class : global___Scalar.Model.ModelClass.V = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal[u"arguments",b"arguments",u"model_class",b"model_class",u"named_arguments",b"named_arguments"]) -> None: ...

    UUID_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    DOC_FIELD_NUMBER: builtins.int
    SPEC_FIELD_NUMBER: builtins.int
    PROPERTIES_FIELD_NUMBER: builtins.int
    uuid: typing.Text = ...
    """A Scalar does not ensure this possibility. As a consequence, oprations
    from standard libraries are allowed (pandas.mean, numpy.std,...) but
    operations implemented for Datasets by Sarus like computing marginals or
    fitting a Keras model cannot be performed on a Scalar.

    Scalars are generated by transforms that explicitly require a specific
    format (e.g. as_pandas, as_numpy,...) or as byproducts of transforms
    (model weights, training history,...).

    e.g. RFC 4122 id used to refer to the dataset (content linked?)
    """

    name: typing.Text = ...
    doc: typing.Text = ...
    @property
    def spec(self) -> global___Scalar.Spec: ...
    @property
    def properties(self) -> google.protobuf.internal.containers.ScalarMap[typing.Text, typing.Text]:
        """Other properties"""
        pass
    def __init__(self,
        *,
        uuid : typing.Text = ...,
        name : typing.Text = ...,
        doc : typing.Text = ...,
        spec : typing.Optional[global___Scalar.Spec] = ...,
        properties : typing.Optional[typing.Mapping[typing.Text, typing.Text]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal[u"spec",b"spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal[u"doc",b"doc",u"name",b"name",u"properties",b"properties",u"spec",b"spec",u"uuid",b"uuid"]) -> None: ...
global___Scalar = Scalar
